/**************************************************************************************
Author:         Dave Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:		Notes.ms
Date:			5/24/11
Description:	This script will store and recall user defined information stored in
                the scene
***************************************************************************************/

try (destroyDialog scene_notes_rollout) catch()


struct SceneNotes
(
    /**************************************************************************
    Returns the custom attribute for saving scene notes
    **************************************************************************/
    fn get_note_custom_attribute =
    (
        note_attr = attributes "sceneNotes"
                    (
                        parameters params
                        (
                            noteNames type:#stringTab tabSizeVariable:true animatable:false
                            noteContents type:#stringTab tabSizeVariable:true animatable:false
                        ) --end parameters
                    ) --end attributes

        note_attr --return value
    ), --end fn

    /**************************************************************************
    Sets the root node note custom attribute. Returns true on success
    **************************************************************************/
    fn set_note_custom_attribute =
    (
        local result = true

        if not (isProperty rootNode "sceneNotes") do
        (
            custom_attr = this.get_note_custom_attribute()

            if not (custAttributes.add rootNode custom_attr) do
                result = false
        ) --end if

        result --return value
    ), --end fn

    /**************************************************************************
    Saves the current note to a text file. Returns true on success
    **************************************************************************/
    fn write_note_to_file file_path note_contents =
    (
        local note_file = openFile file_path mode:"w"
        local result = true

		if note_file != undefined do
		(
			format "%" note_contents to:note_file
			close note_file
		)--end if

        result --return value
    ), --end fn

    /**************************************************************************
    Reads a note file. Returns the note contents on success, "READERROR" on
    failure
    **************************************************************************/
    fn read_note_file file_path =
    (
        local note_file = openFile file_path mode:"r"
		local note_contents = "READERROR"
		local max_length = undefined

		if note_file != undefined do
		(
			seek note_file #eof
			max_length = filepos note_file
			seek note_file 0
			note_contents = readChars note_file max_length errorAtEOF:false
			close note_file
		)--end if

        note_contents --return value
    ) --end fn
) --end struct


( --local scope
    global scene_notes_rollout
	local scene_notes_ini = getDir #plugcfg + "\\SceneNotes.ini"
	local scene_notes_position

    rollout scene_notes_rollout "Notes" width:290 height:420
    (
        local scene_notes = SceneNotes()
        local current_note_index = 1

        /***********************************************************************
        Enables/Disables the UI controls based on argument 'state'
        ***********************************************************************/
        fn enable_controls state =
        (
            scene_notes_rollout.note_name_dropdown.enabled = state
            scene_notes_rollout.note_content_text.enabled = state
            scene_notes_rollout.save_note_button.enabled = state
            scene_notes_rollout.delete_note_button.enabled = state
            scene_notes_rollout.delete_all_button.enabled = state
        ) --end fn

        /***********************************************************************
        Creates a new note and appends it to the list. The previous note is
        saved to the scene. Returns true on success
        ***********************************************************************/
        fn create_note note_name note_content =
        (
            --CRITICAL! Validate root node custom variable
            if not (scene_notes.set_note_custom_attribute()) do
                return false

            --Save current note
            if scene_notes_rollout.note_name_dropdown.selection != 0 do
                rootNode.sceneNotes.noteContents[current_note_index] = scene_notes_rollout.note_content_text.text

            --Create new note
            append rootNode.sceneNotes.noteNames note_name
            append rootNode.sceneNotes.noteContents note_content

            --Refresh UI
            scene_notes_rollout.note_name_dropdown.items = join scene_notes_rollout.note_name_dropdown.items #(note_name)
            scene_notes_rollout.note_name_dropdown.selection = scene_notes_rollout.note_name_dropdown.items.count
            scene_notes_rollout.note_content_text.text = note_content

            enable_controls true

            --CRITICAL! Store new current note index
            current_note_index = scene_notes_rollout.note_name_dropdown.selection

            true --return value
        ) --end fn

        /***********************************************************************
        Deletes the current note from the scene and UI
        ***********************************************************************/
        fn delete_note note_index =
        (
            --Delete note
            deleteItem rootNode.sceneNotes.noteNames note_index
            deleteItem rootNode.sceneNotes.noteContents note_index

            --Refresh UI
            new_list = scene_notes_rollout.note_name_dropdown.items
            deleteItem new_list note_index
            scene_notes_rollout.note_name_dropdown.items = new_list

            if new_list.count > 0 then
            (
                if note_index > new_list.count do
                    note_index = new_list.count

                scene_notes_rollout.note_name_dropdown.selection = note_index
                scene_notes_rollout.note_content_text.text = rootNode.sceneNotes.noteContents[note_index]

                --CRITICAL! Store new current note index
                current_note_index = note_index
            ) --end if
            else
            (
                scene_notes_rollout.note_content_text.text = ""

                enable_controls false
            ) --end else
        ) --end fn

        /***********************************************************************
        UI creation
        ***********************************************************************/
        groupBox new_group "New Note" pos:[5,5] width:280 height:46
            label title_label "Title:" pos:[15,25] width:30 height:15
            editText note_name_text "" pos:[45,24] width:175 height:17
            button create_note_button "Create" pos:[225,23] width:50 height:19

        groupBox save_group "Saved Notes" pos:[5,55] width:280 height:260
            dropDownList note_name_dropdown "" pos:[15,75] width:260 height:21
            editText note_content_text "" pos:[13,105] width:262 height:200

        groupBox option_group "Options" pos:[5,320] width:280 height:75
            button save_note_button "Save Note to File" pos:[15,340] width:125 height:20
            button load_note_button "Load Note from File" pos:[15,365] width:125 height:20
            button delete_note_button "Delete Current Note" pos:[150,340] width:125 height:20
            button delete_all_button "Delete All Notes" pos:[150,365] width:125 height:20

        label status_label "INFO: Standing by..." pos:[5,402] width:280 height:15

        /***********************************************************************
        Rollout open event handler. Prepares the rollout for first time open
        ***********************************************************************/
        on scene_notes_rollout open do
        (
            if (isProperty rootNode "sceneNotes") then
            (
                note_name_dropdown.items = rootNode.sceneNotes.noteNames as array

                if rootNode.sceneNotes.noteContents.count > 0 do
                    note_content_text.text = rootNode.sceneNotes.noteContents[1]
            ) --end if
            else
                enable_controls false
        ) --end on

        /***********************************************************************
        Rollout moved event handler. Stores the current position of the rollout
        ***********************************************************************/
        on scene_notes_rollout moved pos do
            setIniSetting scene_notes_ini "Dialog" "Position" (pos as string)

        /***********************************************************************
        Rollout close event handler. Saves the current note before closing. NOTE:
        If all notes have been deleted then it removes the scene notes custom
        variable from the root note
        ***********************************************************************/
        on scene_notes_rollout close do
        (
            if note_name_dropdown.items.count > 0 then
                rootNode.sceneNotes.noteContents[note_name_dropdown.selection] = note_content_text.text
            else
            (
                for i = rootNode.custAttributes.count to 1 by -1 do
                (
                    if rootNode.custAttributes[i].name == "sceneNotes" do
                        custAttributes.delete rootNode i
                ) --end for
            ) --end else
        ) --end on

        /***********************************************************************
        Rollout close event handler. S
        ***********************************************************************/
        on create_note_button pressed do
        (
            if note_name_text.text == "" do
            (
                status_label.text = "ERROR: Note name is blank"
                return false
            ) --end if

            if (create_note note_name_text.text "") then
            (
                note_name_text.text = ""
                status_label.text = "INFO: New note created"
            ) --end if
            else
                status_label.text = "ERROR: Could not create note"
        ) --end on

        /***********************************************************************
        Note changed event handler. Saves the previous note to the scene and
        loads the new note
        ***********************************************************************/
        on note_name_dropdown selected note_index do
        (
            if note_index == current_note_index do
                return false

            rootNode.sceneNotes.noteContents[current_note_index] = note_content_text.text
            current_note_index = note_index
            note_content_text.text = rootNode.sceneNotes.noteContents[note_index]

            status_label.text = "INFO: Previous note saved. New note loaded"
        ) --end on

        /***********************************************************************
        Save note event handler. Saves the current note to a text file
        ***********************************************************************/
        on save_note_button pressed do
        (
            local file_path = getSaveFileName caption:"Save Note to File" filename:(note_name_dropdown.selected + ".mxn") types:"Max Note (*.mxn)"
            local message = "WARNING: Save note cancelled by user"

			if file_path != undefined then
            (
                if (scene_notes.write_note_to_file file_path note_content_text.text) then
                    message = "INFO: Note saved to file"
                else
                    message = "ERROR: Could not save note to file"
            ) --end if

            status_label.text = message
        ) --end on

        /***********************************************************************
        Load note event handler. Loads the note contents from a file
        ***********************************************************************/
        on load_note_button pressed do
        (
            local file_path = getOpenFileName caption:"Read Note from File" filename:"" types:"Max Note (*.mxn)"
            local note_contents = ""
            local message = "WARNING: Load note cancelled by user"

			if file_path != undefined do
            (
                note_contents = scene_notes.read_note_file file_path

                if note_contents != "READERROR" then
                (
                    if (create_note (getFilenameFile file_path) note_contents) then
                        message = "INFO: Note loaded from file"
                    else
                        message = "ERROR: Could not save loaded note to scene"
                ) --end if
                else
                    message = "ERROR: Could not load note from file"
            ) --end if

            status_label.text = message
        ) --end on

        /***********************************************************************
        Delete note event handler. Deletes the current note from the scene
        ***********************************************************************/
        on delete_note_button pressed do
        (
            if note_name_dropdown.selection == 0 do
                return false

            delete_note note_name_dropdown.selection

            status_label.text = "INFO: Note deleted"
        ) --end on

        /***********************************************************************
        Delete all event handler. Deletes all notes from the scene
        ***********************************************************************/
        on delete_all_button pressed do
        (
            if (queryBox "Are you sure you want to delete all notes?" title:"Delete Confirmation") then
            (
                if note_name_dropdown.selection == 0 do
                    return false

                for i = note_name_dropdown.items.count to 1 by -1 do
                    delete_note i

                status_label.text = "INFO: All notes deleted"
            ) --end if
            else
                status_label.text = "WARNING: Delete all notes cancelled"
        ) --end on
    ) --end rollout

    scene_notes_position = execute (getIniSetting scene_notes_ini "Dialog" "Position")
	if scene_notes_position == OK do scene_notes_position = [100,100]
	if scene_notes_position.x > sysinfo.desktopsize.x - 290 do
        scene_notes_position.x = sysinfo.desktopsize.x - 295
	if scene_notes_position.y > sysinfo.desktopsize.y - 480
        do scene_notes_position.y = sysinfo.desktopsize.y - 480
	if scene_notes_position.x < 0 do scene_notes_position.x = 0
	if scene_notes_position.y < 0 do scene_notes_position.y = 0
	createDialog scene_notes_rollout 290 420 scene_notes_position.x scene_notes_position.y style:#(#style_titlebar, #style_minimizebox, #style_border, #style_sysmenu)
) --end scope
